@@ -32,10 +32,10 @@
 #include	"md_p.h"
 
 int Create(struct supertype *st, char *mddev, int mdfd,
-	   int chunk, int level, int layout, unsigned long size, int raiddisks, int sparedisks,
+	   int chunk, int level, int layout, unsigned long long size, int raiddisks, int sparedisks,
 	   char *name,
 	   int subdevs, mddev_dev_t devlist,
-	   int runstop, int verbose, int force,
+	   int runstop, int verbose, int force, int assume_clean,
 	   char *bitmap_file, int bitmap_chunk, int write_behind, int delay)
 {
 	/*
@@ -69,9 +69,13 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 	int vers;
 	int rv;
 	int bitmap_fd;
+	unsigned long long bitmapsize;
 
 	mdu_array_info_t array;
-	
+	int major = BITMAP_MAJOR_HI;
+
+	memset(&array, 0, sizeof(array));
+
 	vers = md_get_version(mdfd);
 	if (vers < 9000) {
 		fprintf(stderr, Name ": Create requires md driver version 0.90.0 or later\n");
@@ -210,6 +214,27 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 			ldsize = dsize;
 			ldsize <<= 9;
 		}
+		if (st == NULL) {
+			/* Need to choose a default metadata, which is different
+			 * depending on the sizes of devices
+			 */
+			int i;
+			char *name = "default";
+			if (level >= 1 && ldsize > (0x7fffffffULL<<10))
+				name = "default/large";
+			for(i=0; !st && superlist[i]; i++)
+				st = superlist[i]->match_metadata_desc(name);
+
+			if (!st) {
+				fprintf(stderr, Name ": internal error - no default metadata style\n");
+				exit(2);
+			}
+			if (st->ss->major != 0 ||
+			    st->minor_version != 90)
+				fprintf(stderr, Name ": Defaulting to verion %d.%d metadata\n",
+					st->ss->major,
+					st->minor_version);
+		}
 		freesize = st->ss->avail_size(st, ldsize >> 9);
 		if (freesize == 0) {
 			fprintf(stderr, Name ": %s is too small: %luK\n",
@@ -220,10 +245,14 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 		}
 
 		freesize /= 2; /* convert to K */
+		if (chunk) {
+			/* round to chunk size */
+			freesize = freesize & ~(chunk-1);
+		}
 
 		if (size && freesize < size) {
 			fprintf(stderr, Name ": %s is smaller that given size."
-				" %lluK < %luK + superblock\n", dname, freesize, size);
+				" %lluK < %lluK + superblock\n", dname, freesize, size);
 			fail = 1;
 			close(fd);
 			continue;
@@ -254,18 +283,18 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 		}
 		if (level > 0 || level == LEVEL_MULTIPATH || level == LEVEL_FAULTY) {
 			/* size is meaningful */
-			if (minsize > 0x100000000ULL) {
+			if (minsize > 0x100000000ULL && st->ss->major == 0) {
 				fprintf(stderr, Name ": devices too large for RAID level %d\n", level);	
 				return 1;
 			}
 			size = minsize;
 			if (verbose > 0)
-				fprintf(stderr, Name ": size set to %luK\n", size);
+				fprintf(stderr, Name ": size set to %lluK\n", size);
 		}
 	}
 	if (level > 0 && ((maxsize-size)*100 > maxsize)) {
 		if (runstop != 1 || verbose >= 0)
-			fprintf(stderr, Name ": largest drive (%s) exceed size (%luK) by more than 1%%\n",
+			fprintf(stderr, Name ": largest drive (%s) exceed size (%lluK) by more than 1%%\n",
 				maxdisc, size);
 		warn = 1;
 	}
@@ -286,7 +315,7 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 	 * as missing, so that a reconstruct happens (faster than re-parity)
 	 * FIX: Can we do this for raid6 as well?
 	 */
-	if (force == 0 && first_missing >= raiddisks) {
+	if (assume_clean==0 && force == 0 && first_missing >= raiddisks) {
 		switch ( level ) {
 		case 5:
 			insert_point = raiddisks-1;
@@ -316,11 +345,24 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 	       (insert_point < raiddisks || first_missing < raiddisks) )
 	     ||
 	     ( level == 6 && missing_disks == 2)
+	     ||
+	     assume_clean
 		)
 		array.state = 1; /* clean, but one+ drive will be missing */
 	else
 		array.state = 0; /* not clean, but no errors */
 
+	if (level == 10) {
+		/* for raid10, the bitmap size is the capacity of the array,
+		 * which is array.size * raid_disks / ncopies;
+		 * .. but convert to sectors.
+		 */
+		int ncopies = (layout>>8) * (layout & 255);
+		bitmapsize = (unsigned long long)size * raiddisks / ncopies * 2;
+/*		printf("bms=%llu as=%d rd=%d nc=%d\n", bitmapsize, size, raiddisks, ncopies);*/
+	} else
+		bitmapsize = (unsigned long long)size * 2;
+
 	/* There is lots of redundancy in these disk counts,
 	 * raid_disks is the most meaningful value
 	 *          it describes the geometry of the array
@@ -345,18 +387,26 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 	array.nr_disks = array.working_disks + array.failed_disks;
 	array.layout = layout;
 	array.chunk_size = chunk*1024;
+	array.major_version = st->ss->major;
 
-
-	if (!st->ss->init_super(st, &super, &array, name))
+	if (!st->ss->init_super(st, &super, &array, size, name))
 		return 1;
 
+	if (bitmap_file && vers < 9003) {
+		major = BITMAP_MAJOR_HOSTENDIAN;
+#ifdef __BIG_ENDIAN
+		fprintf(stderr, Name ": Warning - bitmaps created on this kernel are not portable\n"
+			"  between different architectured.  Consider upgrading the Linux kernel.\n");
+#endif
+	}
+
 	if (bitmap_file && strcmp(bitmap_file, "internal")==0) {
 		if ((vers%100) < 2) {
 			fprintf(stderr, Name ": internal bitmaps not supported by this kernel.\n");
 			return 1;
 		}
 		if (!st->ss->add_internal_bitmap(st, super, bitmap_chunk, delay, write_behind,
-						 &array.size, 1)) {
+						 bitmapsize, 1, major)) {
 			fprintf(stderr, Name ": Given bitmap chunk size not supported.\n");
 			return 1;
 		}
@@ -387,14 +437,15 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 
 		st->ss->uuid_from_super(uuid, super);
 		if (CreateBitmap(bitmap_file, force, (char*)uuid, bitmap_chunk,
-			delay, write_behind,
-				 array.size*2ULL /* FIXME wrong for raid10 */)) {
+				 delay, write_behind,
+				 bitmapsize,
+				 major)) {
 			return 1;
 		}
 		bitmap_fd = open(bitmap_file, O_RDWR);
 		if (bitmap_fd < 0) {
 			fprintf(stderr, Name ": weird: %s cannot be openned\n",
-			       bitmap_file);
+				bitmap_file);
 			return 1;
 		}
 		if (ioctl(mdfd, SET_BITMAP_FILE, bitmap_fd) < 0) {
@@ -451,6 +502,8 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 				break;
 			case 2:
 				if (disk.state == 1) break;
+				Kill(dv->devname, 0, 1); /* Just be sure it is clean */
+				Kill(dv->devname, 0, 1); /* and again, there could be two superblocks */
 				st->ss->write_init_super(st, super, &disk, dv->devname);
 
 				if (ioctl(mdfd, ADD_NEW_DISK, &disk)) {
